;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x32f09845a958f41eb4e8b6a0613c5682 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x6ea54251b4e6498d524a306d89ac3286 . "./code.cg2js/cg-printing.fasl")
(#x8885c93f9031203f680d1485b55c9eb6 . "./code.cg2js/cg-caret.fasl")
(#xead8a380b761a5341dac0c56b77f6ad7 . "./code.cg2js/cg-item-list.fasl")
(#x6073cc9e48747da223939661cedc579e . "./code.cg2js/cg-progress-indicator.fasl")
(#xb26d1dd64437ba1c28f3f74d859c5968 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x01579c8252e0b4102e0bf547ff9e83f6 . "./code.cg2js/cg-outline.fasl")
(#x50b44b475cab3f8d039fab2a9022ab87 . "./code.cg2js/cg-palette.fasl")
(#x749802a48a08a7285ebfd4579d9d147b . "./code.cg2js/cg-selected-object.fasl")
(#x6085b2f0c3ae6fd9d3a391044f974027 . "./code.cg2js/cg-status-bar.fasl")
(#xd9905ebba437ca3abfa8b1d2e0c686d0 . "./code.cg2js/cg-wrap-string.fasl")
(#x74e22f1612cb11dd2097f75e3bf3eda6 . "./code.cg2js/cg-fill-texture.fasl")
(#xbc7006958130954c5ea62d4325b1bce5 . "./code.cg2js/cg-layout.fasl")
(#xcc7fd89bd6bab3d7e1f7595b91f1de59 . "./code.cg2js/cg-timer.fasl")
(#x05bff89acc0ae059eeb4b31954689ab8 . "./code.cg2js/cg-dialog-item.fasl")
(#x0a3b91ee5a98bbf83dd652a929b3cc96 . "./code.cg2js/cg-button.fasl")
(#x30e0cf75366f42e93b841dbc196be21d . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x13ab9460863e70041bf083777e666a42 . "./code.cg2js/cg-tray.fasl")
(#xa0fd4e9d5b7e6501f4384441e49beb9c . "./code.cg2js/cg-plot-widget.fasl")
(#x826ff2348e726de4d3f1600d3a50e852 . "./code.cg2js/cg-pixmap.fasl")
(#xc17f9340128a7fe561f7f74983e7a12e . "./code.cg2js/cg-font-dialog.fasl")
(#xe7878018bf95d492a1197271a498743c . "./code.cg2js/cg-os-widget.fasl")
(#xf882e76024ca21db2018fe5df5d2550b . "./code.cg2js/cg-text-or-combo.fasl")
(#x287d0a6a659472f89b6956f829290c05 . "./code.cg2js/cg-trackbar.fasl")
(#xbd9eab2a6d17a5fe8ba5fdd6d9afb91b . "./code.cg2js/cg-up-down-control.fasl")
(#xe50c23013e2bd3954063921a0a957d71 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x0cb386bc6455682403dc0ae35c1cdf40 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xb40b25932df0c6463e8c724538d46ae8 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x298bebfbdc699214430c3ff2d50e3351 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x8681be9429fe78ce45410301cf65d45c . "./code.cg2js/cg-chart-widget.fasl")
(#x1362a120b303855e46cb0d20e8d72176 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xf61aa3d156923fca70b45f245e366083 . "./code.cg2js/cg-group-box.fasl")
(#x4535ab14ba3a69993aef0234270d9b94 . "./code.cg2js/cg-static-text.fasl")
(#x5c1fca6cbe884ea80eea90dd0172644a . "./code.cg2js/cg-class-slot-grid.fasl")
(#x87860d939869c438b0f50658ab072bf3 . "./code.cg2js/cg-color-dialog.fasl")
(#x501e26dcfbbeb433687bc0ca0e569332 . "./code.cg2js/cg-intersect.fasl")
(#x7b02ff6f3838e471dd1333c4f847d393 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x51a7271461f47041991a5bf76dc2832f . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xae6bc455ba212755c85cfc4b769c5251 . "./code.cg2js/cg-html-dialog.fasl")
(#xa3f97f934d9112b37d103abd2c699c37 . "./code.cg2js/cg-lettered-menu.fasl")
(#x7eb8c1055fbea15b4a14dee9da4084b0 . "./code.cg2js/cg-class-grid.fasl")
(#x27742ee19fed0ad3d4dbb4ed42114b35 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x742107a95a0302c701aa07016cf3400c . "./code.cg2js/cg-calendar.fasl")
(#xff250e537c140aec8466b360aa76b404 . "./code.cg2js/cg-editable-text.fasl")
(#x79ac441e044d41c50a3712b5dd9fb4c8 . "./code.cg2js/cg-property.fasl")
(#x87efe6c5d820114ef591c419597f8d0f . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x28668d4ac67f2694fcc8dc462d652ab5 . "./code.cg2js/cg-grid-widget.fasl")
(#x283a02d06258b6ee3299eeb1f5487417 . "./code.cg2js/cg-paren-matching.fasl")
(#xd772d1474b6d8005032f0b7c30d055f5 . "./code.cg2js/cg-dev.fasl")
(#xb5d1b7cb26fa801cbf8aa0c43469c8bd . "./code.cg2js/cg-drawable.fasl")
(#xb614b5fda053b5c71bc5b60829105aef . "./code.cg2js/cg-curve.fasl")
(#x4e9445e5c89284e1ba0832a713959120 . "./code.cg2js/cg-lisp-widget.fasl")
(#xc5dd8a0faefaa0f8cdea678bbbda640c . "./code.cg2js/cg-dde-utils.fasl")
(#x66ee485eda5f38c264c7cc7cebaf6c6b . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xfd29418a76d57ef736add5c38f517d56 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xc1a8906ee99ce5e9478495f90f01be7e . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x9f93526b6968d9a9d18b3869cc214676 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xd85a809ffd27fd863eb43fa7486ac894 . "./code.cg2js/cg-lamp.fasl")
(#x38e94ad26f76c60d0c6dfdf53fbd21d8 . "./code.cg2js/cg-common-control.fasl")
(#xfc3e681562206bc233d19c823c65f6d0 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xb3c8db8aff4f03d1127a887d1928889d . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x6d566580433931de9a660b63c6c4f2c3 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xa735a9cec7fbb285d075b5e66c9f9712 . "./code.cg2js/cg-project-window.fasl")
(#xabdd7a4448e3d78125c9e37a156eaae2 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x7eb0529a70d8216fae8b1e6b66b8840c . "./code.cg2js/cg-class-support.fasl")
(#x4563a297decd636e4b42a6d9ee731c65 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x532daac746485bc6090de117bb724778 . "./code.cg2js/cg-yes-no-list.fasl")
(#x639e8e4a9a26ab1cf95dc620212398b7 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xceef6e0b229c712147e11d94d6998297 . "./code.cg2js/cg-string-dialog.fasl")
(#x828a4ba2d952ff789038661bf375acc4 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xb7a22f39a3efab07736b5a5492d422dd . "./code.cg2js/cg-dropping-outline.fasl")
(#xbc2cf5377927e30ede64ead99f9e5b2f . "./code.cg2js/cg.fasl")
(#x7302f5a872dd558331b68c94b1ab740b . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x84ce19364c84f8f0efa6d42c8acb84e3 . "./code.cg2js/cg-scaling-stream.fasl")
(#xd3b3b88c59ca9e62f9df2575f933bdab . "./code.cg2js/cg-yes-no-string.fasl")
(#xc534171c449a784744469b188382e12a . "./code.cg2js/cg-icon-pixmap.fasl")
(#xc5cbcaa09ff24d8ca2004ac7611d591f . "./code.cg2js/cg-web-browser.fasl")
(#x9e4e7179d4a263833bba83c923a5eae2 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x5b19f413f625fba1c79f7a23f65ab597 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x26425c3fd8bc457da12160eddf0f571e . "./code.cg2js/cg-message-dialog.fasl")
(#x8b7efc437908d07ebd6d87fb888b4255 . "./code.cg2js/cg-comtab.fasl")
(#xf8c935a755b0949a9e53036c9eeedc57 . "./code.cg2js/cg-base.fasl")
(#x8c70a56f497b1fccc956ff717f8e112b . "./code.cg2js/cg-pixmap-widget.fasl")
(#xdf4587d7f8192cdd6b6ae5dba2066eb0 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xcca060c39e4d4098943ef6c16338156b . "./code.cg2js/cg-utility-dialog.fasl")
(#xfcbea5e94d6fc72ca589e2a276563f60 . "./code.cg2js/cg-object-editor.fasl")
(#x20f0f5cdb82e7f8f13a1f46e5b14f7c9 . "./code.cg2js/cg-edit-in-place.fasl")
(#xfe443f1310eadb287b9f00e98ffe0f2c . "./code.cg2js/cg-radio-button.fasl")
(#xde748e2f94e456dd7d75581db77b9b2e . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x09aaab97cb1aabc06e1ab8ccda333440 . "./code.cg2js/cg-nodes.fasl")
(#xed46565202502f1538783272974b495f . "./code.cg2js/cg-get-position.fasl")
(#x8ef828be133b7bceec1ab06f196c756a . "./code.cg2js/cg-directory-dialog.fasl")
(#xfd0d09d22f2166a7b202210244d4f728 . "./code.cg2js/cg-lisp-text.fasl")
(#xb4a7410ba6f1203d0b27b20bece3b592 . "./code.cg2js/cg-combo-box.fasl")
(#xc60f42d50a15c989cedeadef884ec2ff . "./code.cg2js/cg-text-widget.fasl")
(#x13b15ee55c816ac6b2c0ceb1988772c6 . "./code.cg2js/cg-list-view.fasl")
(#x622d52b36d6244ab84a6f4cdb86d31a9 . "./code.cg2js/cg-graphics-context.fasl")
(#xdb0d667d23522e1bda8f0b1569902a1b . "./code.cg2js/cg-get-pixmap.fasl")
(#x63190aa02b95b0a76feeef4a74d79dd9 . "./code.cg2js/cg-menu.fasl")
(#xb5f4944f9bf78302391d6fe8bd51a7a6 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xd8bc822da4d6a1142be294ca90c434a3 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x3c584298ac1790ff36a238e7e9a3f77f . "./code.cg2js/cg-tooltip.fasl")
(#x17b03d781b4deaa87a138b353e46a76c . "./code.cg2js/cg-html-widget.fasl")
(#xe8f476a68f9682c477e2161a95d1218b . "./code.cg2js/cg-choice-list.fasl")
(#x1ea8b252977c01c29023b48ed799db2f . "./code.cg2js/cg-icon.fasl")
(#x4f98a4cc636d26d71d20a7e6991535a5 . "./code.cg2js/cg-acache.fasl")
(#x5d1ab43f88febd340897bfb9ed872017 . "./code.cg2js/cg-template-string.fasl")
(#x147e62ec738f80b183b35161326ac792 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xcb614e5ba657e0af6bf834f940626e6d . "./code.cg2js/cg-toggling-widget.fasl")
(#x2c05cce1427b003e155eea066923fc41 . "./code.cg2js/cg-tab-control.fasl")
(#xcf0c8243172793b3d70d8a7d2d35a58c . "./code.cg2js/cg-split-bar.fasl")
(#x8015f642e7cc0eb0618b27b1be2eafc9 . "./code.cg2js/cg-picture-widget.fasl")
(#x7cc6500423e7e48186327b84daba9252 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xa117092c2190acd2946537bce9d463d9 . "./code.cg2js/cg-choose-printer.fasl")
(#x7dd0dbafaa552c0afb737d768c9bc41f . "./code.cg2js/cg-os-window.fasl")
(#xdfe8e95e79d3de2c89a75861e5fc9183 . "./code.cg2js/cg-scroll-bar.fasl")
(#xff80e0a72c441110c77798c813561adf . "./code.cg2js/cg-clipboard.fasl")
(#x6bc48d5404e3a89767c90ebd2c2d639c . "./code.cg2js/cg-find-string-dialog.fasl")
(#x1f6dffa8c76a20b04767ffcaf851cb9d . "./code.cg2js/cg-hotspot.fasl")
(#x2c04e96266893ff571b95eac881d3cec . "./code.cg2js/cg-check-box.fasl")
(#xb8e58e54dc0d5eaf2fa390a37586b635 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x50d0b967c329246a731a2e766c9af0dc . "./code.cg2js/cg-file-dialog.fasl")
(#x61ec387e1b50cb020e661607422eea3a . "./code.cg2js/cg-toolbar.fasl")
(#x33a5e56cde6456fa42d72cfdab6fd5d9 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xa8e796699bcd9bf9735d406f45c0ebab . "./code.cg2js/cg-chart-or-plot.fasl")
(#xba05dbe245ad0e0cfed72984d94d0be1 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x031b84130f063e42d9a11f63560838b2 . "./code.idejs/ide-project.fasl")
(#xd85d37851944471aeaaae8177ff92418 . "./code.idejs/ide-intro.fasl")
(#x701e73f326d277e9e415754fed3cb31a . "./code.idejs/ide-find-in-files.fasl")
(#x987d118d775614520014115303374467 . "./code.idejs/ide-grapher.fasl")
(#x6bfdf306b20bf535f0718234bab10b83 . "./code.idejs/ide-inspector.fasl")
(#x862520b1e66b72371bacc79f9fab4908 . "./code.idejs/ide-editor.fasl")
(#x7395ab6e820ed20fdabbff79d6a86db5 . "./code.idejs/ide-window-list.fasl")
(#x389c7bfdba9bc98c842e3162d592a23f . "./code.idejs/ide-process-browser.fasl")
(#x71f92b6ec4c6a9569190c67e9f21c041 . "./code.idejs/ide-patch-dialog.fasl")
(#xf065377a024597ca66cc6aa0675ab99e . "./code.idejs/ide-class-grapher.fasl")
(#xe9287aff46cbfc60c0d5de778501b9db . "./code.idejs/ide-base.fasl")
(#x276ea187bae73acda2629acb7298ebe4 . "./code.idejs/ide.fasl")
(#x0f6d74916a86243949764a7e5465f960 . "./code.idejs/ide-trace.fasl")
(#x8d9db78b9976a6d29e381a8f3e06fb9a . "./code.idejs/ide-form.fasl")
(#xb5490897d2a0716fcdc1e3d565ed8abc . "./code.idejs/ide-find-definitions.fasl")
(#x29727d5b3387574ad8875f22351cc660 . "./code.idejs/ide-apropos.fasl")
(#xd3d7c3d0edf66bf351d28bd35aba7a4c . "./code.idejs/ide-class-browser.fasl")
(#xa19022fa8e4534144407d2f00b73fb58 . "./code.idejs/ide-navigator.fasl")
(#x6d9a63380404a03044d89ca06b243b78 . "./code.idejs/ide-clipboard.fasl")
(#x794cd0d307e87f70dd9640483ee86fc2 . "./code.idejs/ide-acache.fasl")
(#x763fca1b6e3b2eb92a6b0490ece55a42 . "./code.idejs/ide-keyshort.fasl")
(#x0126bea82f4dcb6cb167b561e00fe45d . "./code.idejs/ide-package-list.fasl")
(#x5e0bbe4b60d5b86cbc10f5ddaba23da8 . "./code.idejs/ide-dev.fasl")
(#x696b64509ebe0538a3b58ac1081d4fa0 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
